@@ -187,6 +187,11 @@ do_hup(void)
         generate_ed_link_cert(options, now, new_signing_key > 0)) {
       log_warn(LD_OR, "Problem reloading Ed25519 keys; still using old keys.");
     }
+    if (load_family_id_keys(options,
+                            networkstatus_get_latest_consensus())) {
+      log_warn(LD_OR, "Problem reloading family ID keys; "
+               "still using old keys.");
+    }
 
     /* Update cpuworker and dnsworker processes, so they get up-to-date
      * configuration options. */
@@ -590,7 +595,7 @@ tor_init(int argc, char *argv[])
 
     log_notice(LD_GENERAL, "Tor can't help you if you use it wrong! "
                "Learn how to be safe at "
-               "https://www.torproject.org/download/download#warning");
+               "https://support.torproject.org/faq/staying-anonymous/");
 
     if (strstr(version, "alpha") || strstr(version, "beta"))
       log_notice(LD_GENERAL, "This version is not a stable Tor release. "
@@ -825,6 +830,39 @@ do_dump_config(void)
   return 0;
 }
 
+/** Implement --keygen-family; create a family ID key and write it to a file.
+ */
+static int
+do_keygen_family(const char *fname_base)
+{
+  ed25519_public_key_t pk;
+  char *fname_key = NULL, *fname_id = NULL, *id_contents = NULL;
+  int r = -1;
+
+  if (BUG(!fname_base))
+    goto done;
+
+  tor_asprintf(&fname_key, "%s.secret_family_key", fname_base);
+  tor_asprintf(&fname_id, "%s.public_family_id", fname_base);
+
+  if (create_family_id_key(fname_key, &pk) < 0)
+    goto done;
+  tor_asprintf(&id_contents, "%s\n", ed25519_fmt(&pk));
+  if (write_str_to_file(fname_id, id_contents, 0) < 0)
+    goto done;
+
+  printf("# Generated %s\n", fname_key);
+  printf("FamilyId %s\n", ed25519_fmt(&pk));
+
+  r = 0;
+
+ done:
+  tor_free(fname_key);
+  tor_free(fname_id);
+  tor_free(id_contents);
+  return r;
+}
+
 static void
 init_addrinfo(void)
 {
@@ -919,11 +957,32 @@ sandbox_init_filter(void)
   OPEN("/etc/hosts");
   OPEN("/proc/meminfo");
 
+#ifdef HAVE_MODULE_RELAY
+  {
+    smartlist_t *family_id_files =
+      list_family_key_files(options, options->FamilyKeyDirectory);
+
+    SMARTLIST_FOREACH(family_id_files, const char *, fn,
+                      OPEN(fn));
+
+    SMARTLIST_FOREACH(family_id_files, char *, cp, tor_free(cp));
+    smartlist_free(family_id_files);
+  }
+#endif
+
   if (options->BridgeAuthoritativeDir)
     OPEN_DATADIR_SUFFIX("networkstatus-bridges", ".tmp");
 
-  if (authdir_mode(options))
+  if (authdir_mode(options)) {
     OPEN_DATADIR("approved-routers");
+    OPEN_DATADIR_SUFFIX("my-consensus-microdesc", ".tmp");
+    OPEN_DATADIR_SUFFIX("my-consensus-ns", ".tmp");
+    if (options->V3BandwidthsFile) {
+      log_notice(LD_GENERAL, "Adding V3BandwidthsFile %s to sandboxing set.",
+                 options->V3BandwidthsFile);
+      OPEN(options->V3BandwidthsFile);
+    }
+  }
 
   if (options->ServerDNSResolvConfFile)
     sandbox_cfg_allow_open_filename(&cfg,
@@ -1001,6 +1060,11 @@ sandbox_init_filter(void)
   if (options->BridgeAuthoritativeDir)
     RENAME_SUFFIX("networkstatus-bridges", ".tmp");
 
+  if (authdir_mode(options)) {
+    RENAME_SUFFIX("my-consensus-microdesc", ".tmp");
+    RENAME_SUFFIX("my-consensus-ns", ".tmp");
+  }
+
 #define STAT_DATADIR(name)                      \
   sandbox_cfg_allow_stat_filename(&cfg, get_datadir_fname(name))
 
@@ -1229,10 +1293,11 @@ run_tor_main_loop(void)
   const time_t now = time(NULL);
   directory_info_has_arrived(now, 1, 0);
 
-  if (server_mode(get_options()) || dir_server_mode(get_options())) {
-    /* launch cpuworkers. Need to do this *after* we've read the onion key. */
-    cpu_init();
-  }
+  /* launch cpuworkers. Need to do this *after* we've read the onion key. */
+  /* launch them always for all tors, now that clients can solve onion PoWs. */
+  if (cpuworker_init() == -1)
+    return -1;
+
   consdiffmgr_enable_background_compression();
 
   /* Setup shared random protocol subsystem. */
@@ -1369,6 +1434,9 @@ tor_run_main(const tor_main_configuration_t *tor_cfg)
   case CMD_KEYGEN:
     result = load_ed_keys(get_options(), time(NULL)) < 0;
     break;
+  case CMD_KEYGEN_FAMILY:
+    result = do_keygen_family(get_options()->command_arg);
+    break;
   case CMD_KEY_EXPIRATION:
     init_keys();
     result = log_cert_expiration();
