@@ -187,6 +187,8 @@ conflux_free_(conflux_t *cfx)
   if (!cfx) {
     return;
   }
+  tor_assert(cfx->legs);
+  tor_assert(cfx->ooo_q);
 
   SMARTLIST_FOREACH_BEGIN(cfx->legs, conflux_leg_t *, leg) {
     SMARTLIST_DEL_CURRENT(cfx->legs, leg);
@@ -260,6 +262,8 @@ unlinked_free(unlinked_circuits_t *unlinked)
   if (!unlinked) {
     return;
   }
+  tor_assert(unlinked->legs);
+
   /* This cfx is pointing to a linked set. */
   if (!unlinked->is_for_linked_set) {
     conflux_free(unlinked->cfx);
@@ -493,10 +497,6 @@ cfx_add_leg(conflux_t *cfx, leg_t *leg)
   /* Big trouble if we add a leg to the wrong set. */
   tor_assert(tor_memeq(cfx->nonce, leg->link->nonce, sizeof(cfx->nonce)));
 
-  if (BUG(CONFLUX_NUM_LEGS(cfx) > CONFLUX_MAX_CIRCS)) {
-    return;
-  }
-
   conflux_leg_t *cleg = tor_malloc_zero(sizeof(*cleg));
   cleg->circ = leg->circ;
   // TODO-329-ARTI: Blindly copying the values from the cell. Is this correct?
@@ -731,6 +731,9 @@ try_finalize_set(unlinked_circuits_t *unlinked)
   bool is_client;
 
   tor_assert(unlinked);
+  tor_assert(unlinked->legs);
+  tor_assert(unlinked->cfx);
+  tor_assert(unlinked->cfx->legs);
 
   /* Without legs, this is not ready to become a linked set. */
   if (BUG(smartlist_len(unlinked->legs) == 0)) {
@@ -738,6 +741,17 @@ try_finalize_set(unlinked_circuits_t *unlinked)
     goto end;
   }
 
+  /* If there are too many legs, we can't link. */
+  if (smartlist_len(unlinked->legs) +
+      smartlist_len(unlinked->cfx->legs) > conflux_params_get_max_legs_set()) {
+    log_fn(LOG_PROTOCOL_WARN, LD_CIRC,
+           "Conflux set has too many legs to link. "
+           "Rejecting this circuit.");
+    conflux_log_set(LOG_PROTOCOL_WARN, unlinked->cfx, unlinked->is_client);
+    err = ERR_LINK_CIRC_INVALID_LEG;
+    goto end;
+  }
+
   /* Validate that all legs are coherent and parameters match. On failure, we
    * teardown the whole unlinked set because this means we either have a code
    * flow problem or the Exit is trying to trick us. */
@@ -797,11 +811,6 @@ try_finalize_set(unlinked_circuits_t *unlinked)
   unlinked->cfx = NULL;
   unlinked_free(unlinked);
 
-  /* Now that this set is ready to use, try any pending streams again. */
-  if (is_client) {
-    connection_ap_attach_pending(1);
-  }
-
   log_info(LD_CIRC,
            "Successfully linked a conflux %s set which is now usable.",
            is_client ? "client" : "relay");
@@ -822,10 +831,29 @@ record_rtt_client(const circuit_t *circ)
   tor_assert(CIRCUIT_IS_ORIGIN(circ));
 
   leg_t *leg = unlinked_leg_find(circ, true);
-  if (leg && leg->link_sent_usec > 0) {
-    leg->rtt_usec = monotime_absolute_usec() - leg->link_sent_usec;
-    return leg->rtt_usec;
+
+  if (BUG(!leg || leg->link_sent_usec == 0)) {
+    log_warn(LD_BUG,
+             "Conflux: Trying to record client RTT without a timestamp");
+    goto err;
   }
+
+  uint64_t now = monotime_absolute_usec();
+  tor_assert_nonfatal(now >= leg->link_sent_usec);
+  leg->rtt_usec = now - leg->link_sent_usec;
+  if (leg->rtt_usec == 0) {
+    log_warn(LD_CIRC, "Clock appears stalled for conflux.");
+    // TODO-329-TUNING: For now, let's accept this case. We need to do
+    // tuning and clean up the tests such that they use RTT in order to
+    // fail here.
+    //goto err;
+  }
+  return leg->rtt_usec;
+
+ err:
+  // Avoid using this leg until a timestamp comes in
+  if (leg)
+    leg->rtt_usec = UINT64_MAX;
   return UINT64_MAX;
 }
 
@@ -842,10 +870,26 @@ record_rtt_exit(const circuit_t *circ)
   tor_assert(CIRCUIT_IS_ORCIRC(circ));
 
   conflux_leg_t *leg = conflux_get_leg(circ->conflux, circ);
-  if (leg && leg->linked_sent_usec > 0) {
-    leg->circ_rtts_usec = monotime_absolute_usec() - leg->linked_sent_usec;
-    return leg->circ_rtts_usec;
+
+  if (BUG(!leg || leg->linked_sent_usec == 0)) {
+    log_warn(LD_BUG,
+             "Conflux: Trying to record exit RTT without a timestamp");
+    goto err;
   }
+
+  uint64_t now = monotime_absolute_usec();
+  tor_assert_nonfatal(now >= leg->linked_sent_usec);
+  leg->circ_rtts_usec = now - leg->linked_sent_usec;
+
+  if (leg->circ_rtts_usec == 0) {
+    log_warn(LD_CIRC, "Clock appears stalled for conflux.");
+    goto err;
+  }
+  return leg->circ_rtts_usec;
+
+ err:
+  if (leg)
+    leg->circ_rtts_usec = UINT64_MAX;
   return UINT64_MAX;
 }
 
@@ -863,6 +907,9 @@ record_rtt(const circuit_t *circ, bool is_client)
   if (is_client) {
     rtt_usec = record_rtt_client(circ);
 
+    if (rtt_usec == UINT64_MAX)
+      return false;
+
     if (rtt_usec >= get_circuit_build_timeout_ms()*1000) {
       log_info(LD_CIRC, "Conflux leg RTT is above circuit build time out "
                         "currently at %f msec. Relaunching.",
@@ -1107,6 +1154,11 @@ conflux_launch_leg(const uint8_t *nonce)
              fmt_nonce(nonce));
   }
 
+  /* Increase the retry count for this conflux object as in this nonce.
+   * We must do this now, because some of the maze's early failure paths
+   * call right back into this function for relaunch. */
+  unlinked->cfx->num_leg_launch++;
+
   origin_circuit_t *circ =
     circuit_establish_circuit_conflux(nonce, CIRCUIT_PURPOSE_CONFLUX_UNLINKED,
                                       exit, flags);
@@ -1136,9 +1188,6 @@ conflux_launch_leg(const uint8_t *nonce)
                                              last_seq_sent, last_seq_recv,
                                              get_client_ux()));
 
-  /* Increase the retry count for this conflux object as in this nonce. */
-  unlinked->cfx->num_leg_launch++;
-
   unlinked_leg_add(unlinked, leg);
   return true;
 
@@ -1287,7 +1336,12 @@ count_client_usable_sets(void)
       log_warn(LD_BUG, "Client conflux linked set leg without a circuit");
       continue;
     }
-    if (!CONST_TO_ORIGIN_CIRCUIT(leg->circ)->unusable_for_new_conns) {
+
+    /* The maze marks circuits used several different ways. If any of
+     * them are marked for this leg, launch a new one. */
+    if (!CONST_TO_ORIGIN_CIRCUIT(leg->circ)->unusable_for_new_conns &&
+        !CONST_TO_ORIGIN_CIRCUIT(leg->circ)->isolation_values_set &&
+        !leg->circ->timestamp_dirty) {
       count++;
     }
   } DIGEST256MAP_FOREACH_END;
@@ -1305,7 +1359,10 @@ conflux_predict_new(time_t now)
 {
   (void) now;
 
-  if (!conflux_is_enabled(NULL)) {
+  /* If conflux is disabled, or we have insufficient consensus exits,
+   * don't prebuild. */
+  if (!conflux_is_enabled(NULL) ||
+      router_have_consensus_path() != CONSENSUS_PATH_EXIT) {
     return;
   }
 
@@ -1558,13 +1615,31 @@ linked_circuit_free(circuit_t *circ, bool is_client)
 {
   tor_assert(circ);
   tor_assert(circ->conflux);
+  tor_assert(circ->conflux->legs);
+  tor_assert(circ->conflux->ooo_q);
+
   if (is_client) {
     tor_assert(circ->purpose == CIRCUIT_PURPOSE_CONFLUX_LINKED);
   }
 
   /* Circuit can be freed without being closed and so we try to delete this leg
    * so we can learn if this circuit is the last leg or not. */
-  cfx_del_leg(circ->conflux, circ);
+  if (cfx_del_leg(circ->conflux, circ)) {
+    /* Check for instances of bug #40870, which we suspect happen
+     * during exit. If any happen outside of exit, BUG and warn. */
+    if (!circ->conflux->in_full_teardown) {
+      /* We should bug and warn if we're not in a shutdown process; that
+       * means we got here somehow without a close. */
+      if (BUG(!shutting_down)) {
+        log_warn(LD_BUG,
+                 "Conflux circuit %p being freed without being marked for "
+                 "full teardown via close, with shutdown state %d. "
+                 "Please report this.", circ, shutting_down);
+        conflux_log_set(LOG_WARN, circ->conflux, is_client);
+      }
+      circ->conflux->in_full_teardown = true;
+    }
+  }
 
   if (CONFLUX_NUM_LEGS(circ->conflux) > 0) {
     /* The last leg will free the streams but until then, we nullify to avoid
@@ -1926,6 +2001,15 @@ conflux_process_linked(circuit_t *circ, crypt_path_t *layer_hint,
     goto end;
   }
 
+  /* If this set is ready to use with a valid conflux set, try any pending
+   * streams again. */
+  if (circ->conflux) {
+    connection_ap_attach_pending(1);
+  }
+
+  /* This cell is now considered valid for clients. */
+  circuit_read_valid_data(TO_ORIGIN_CIRCUIT(circ), cell_len);
+
   goto end;
 
  close:
@@ -2020,14 +2104,93 @@ conflux_pool_init(void)
   }
 }
 
+/**
+ * Return a description of all linked and unlinked circuits associated
+ * with a conflux set.
+ *
+ * For use in rare bug cases that are hard to diagnose.
+ */
+void
+conflux_log_set(int loglevel, const conflux_t *cfx, bool is_client)
+{
+  tor_assert(cfx);
+
+  log_fn(loglevel,
+          LD_BUG,
+           "Conflux %s: %d linked, %d launched. Delivered: %"PRIu64"; "
+           "teardown: %d; Current: %p, Previous: %p",
+               fmt_nonce(cfx->nonce), smartlist_len(cfx->legs),
+               cfx->num_leg_launch,
+               cfx->last_seq_delivered, cfx->in_full_teardown,
+               cfx->curr_leg, cfx->prev_leg);
+
+  // Log all linked legs
+  int legs = 0;
+  CONFLUX_FOR_EACH_LEG_BEGIN(cfx, leg) {
+   const struct congestion_control_t *cc = circuit_ccontrol(leg->circ);
+   log_fn(loglevel, LD_BUG,
+            " - Linked Leg %d purpose=%d; RTT %"PRIu64", sent: %"PRIu64
+            "; sent: %"PRIu64", recv: %"PRIu64", infl: %"PRIu64", "
+            "ptr: %p, idx: %d, marked: %d",
+            legs, leg->circ->purpose, leg->circ_rtts_usec,
+            leg->linked_sent_usec, leg->last_seq_recv,
+            leg->last_seq_sent, cc->inflight, leg->circ,
+            leg->circ->global_circuitlist_idx,
+            leg->circ->marked_for_close);
+   legs++;
+  } CONFLUX_FOR_EACH_LEG_END(leg);
+
+  // Look up the nonce to see if we have any unlinked circuits.
+  unlinked_circuits_t *unlinked = unlinked_pool_get(cfx->nonce, is_client);
+  if (unlinked) {
+    // Log the number of legs and the is_for_linked_set status
+    log_fn(loglevel, LD_BUG, " - Unlinked set:  %d legs, for link: %d",
+             smartlist_len(unlinked->legs), unlinked->is_for_linked_set);
+    legs = 0;
+    SMARTLIST_FOREACH_BEGIN(unlinked->legs, leg_t *, leg) {
+      log_fn(loglevel, LD_BUG,
+        "     Unlinked Leg: %d purpose=%d; linked: %d, RTT %"PRIu64", "
+        "sent: %"PRIu64" link ptr %p, circ ptr: %p, idx: %d, marked: %d",
+               legs, leg->circ->purpose, leg->linked,
+               leg->rtt_usec, leg->link_sent_usec,
+               leg->link, leg->circ,
+               leg->circ->global_circuitlist_idx,
+               leg->circ->marked_for_close);
+      legs++;
+    } SMARTLIST_FOREACH_END(leg);
+  }
+}
+
+/**
+ * Conflux needs a notification when tor_shutdown() begins, so that
+ * when circuits are freed, new legs are not launched.
+ *
+ * This needs a separate notification from conflux_pool_free_all(),
+ * because circuits must be freed before that function.
+ */
+void
+conflux_notify_shutdown(void)
+{
+  shutting_down = true;
+}
+
+#ifdef TOR_UNIT_TESTS
+/**
+ * For unit tests: Clear the shutting down state so we resume building legs.
+ */
+void
+conflux_clear_shutdown(void)
+{
+  shutting_down = false;
+}
+#endif
+
 /** Free and clean up the conflux pool subsystem. This is called by the subsys
  * manager AFTER all circuits have been freed which implies that all objects in
  * the pools aren't referenced anymore. */
 void
 conflux_pool_free_all(void)
 {
-  shutting_down = true;
-
   digest256map_free(client_linked_pool, free_conflux_void_);
   digest256map_free(server_linked_pool, free_conflux_void_);
   digest256map_free(client_unlinked_pool, free_unlinked_void_);
