@@ -386,21 +386,17 @@ protocol_list_supports_protocol_or_later(const char *list,
 /*
  * XXX START OF HAZARDOUS ZONE XXX
  */
-/* All protocol version that this relay version supports. */
+/* All protocol version that this version of tor supports. */
 #define PR_CONFLUX_V   "1"
 #define PR_CONS_V      "1-2"
-#define PR_DESC_V      "1-3"
+#define PR_DESC_V      "1-4"
 #define PR_DIRCACHE_V  "2"
 #define PR_FLOWCTRL_V  "1-2"
 #define PR_HSDIR_V     "2"
 #define PR_HSINTRO_V   "4-5"
 #define PR_HSREND_V    "1-2"
 #define PR_LINK_V      "1-5"
-#ifdef HAVE_WORKING_TOR_TLS_GET_TLSSECRETS
-#define PR_LINKAUTH_V  "1,3"
-#else
 #define PR_LINKAUTH_V  "3"
-#endif
 #define PR_MICRODESC_V "1-3"
 #define PR_PADDING_V   "2"
 #define PR_RELAY_V     "2-4"
@@ -521,8 +517,8 @@ protover_get_supported_protocols(void)
 const char *
 protover_get_recommended_client_protocols(void)
 {
-  return "Cons=2 Desc=2 DirCache=2 HSDir=2 HSIntro=4 HSRend=2 "
-         "Link=4-5 Microdesc=2 Relay=2";
+  return "Cons=2 Desc=2 DirCache=2 FlowCtrl=1-2 HSDir=2 HSIntro=4 HSRend=2 "
+         "Link=4-5 Microdesc=2 Relay=2-4";
 }
 
 /** Return the recommended relay protocols list that directory authorities
@@ -530,8 +526,8 @@ protover_get_recommended_client_protocols(void)
 const char *
 protover_get_recommended_relay_protocols(void)
 {
-  return "Cons=2 Desc=2 DirCache=2 HSDir=2 HSIntro=4 HSRend=2 "
-         "Link=4-5 LinkAuth=3 Microdesc=2 Relay=2";
+  return "Cons=2 Desc=2 DirCache=2 FlowCtrl=1-2 HSDir=2 HSIntro=4-5 HSRend=2 "
+         "Link=4-5 LinkAuth=3 Microdesc=2 Relay=2-4";
 }
 
 /** Return the required client protocols list that directory authorities
@@ -539,7 +535,7 @@ protover_get_recommended_relay_protocols(void)
 const char *
 protover_get_required_client_protocols(void)
 {
-  return "Cons=2 Desc=2 Link=4 Microdesc=2 Relay=2";
+  return "Cons=2 Desc=2 FlowCtrl=1 Link=4 Microdesc=2 Relay=2";
 }
 
 /** Return the required relay protocols list that directory authorities
@@ -547,8 +543,8 @@ protover_get_required_client_protocols(void)
 const char *
 protover_get_required_relay_protocols(void)
 {
-  return "Cons=2 Desc=2 DirCache=2 HSDir=2 HSIntro=4 HSRend=2 "
-         "Link=4-5 LinkAuth=3 Microdesc=2 Relay=2";
+  return "Cons=2 Desc=2 DirCache=2 FlowCtrl=1-2 HSDir=2 HSIntro=4-5 HSRend=2 "
+         "Link=4-5 LinkAuth=3 Microdesc=2 Relay=2-4";
 }
 
 /*
