@@ -1,7 +1,7 @@
 /* Copyright (c) 2001, Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2020, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -33,6 +33,7 @@
 #define LOG_PRIVATE
 #include "lib/log/log.h"
 #include "lib/log/log_sys.h"
+#include "lib/log/util_bug.h"
 #include "lib/version/git_revision.h"
 #include "lib/log/ratelim.h"
 #include "lib/lock/compat_mutex.h"
@@ -706,6 +707,8 @@ log_fn_(int severity, log_domain_mask_t domain, const char *fn,
         const char *format, ...)
 {
   va_list ap;
+  if (domain & LD_BUG)
+    tor_bug_increment_count_();
   if (severity > log_global_min_severity_)
     return;
   va_start(ap,format);
@@ -718,6 +721,8 @@ log_fn_ratelim_(ratelim_t *ratelim, int severity, log_domain_mask_t domain,
 {
   va_list ap;
   char *m;
+  if (domain & LD_BUG)
+    tor_bug_increment_count_();
   if (severity > log_global_min_severity_)
     return;
   m = rate_limit_log(ratelim, approx_time());
@@ -784,7 +789,7 @@ logs_free_all(void)
 
 /** Flush the signal-safe log files.
  *
- * This function is safe to call from a signal handler. It is currenly called
+ * This function is safe to call from a signal handler. It is currently called
  * by the BUG() macros, when terminating the process on an abnormal condition.
  */
 void
@@ -912,6 +917,7 @@ init_logging(int disable_startup_queue)
 {
   if (!log_mutex_initialized) {
     tor_mutex_init(&log_mutex);
+    tor_bug_init_counter();
     log_mutex_initialized = 1;
   }
 #ifdef __GNUC__
