@@ -588,6 +588,10 @@ tor_tls_write(tor_tls_t *tls, const char *cp, size_t n)
   tor_assert(cp || n == 0);
   tor_assert(n < INT_MAX);
 
+  if (n == 0) {
+    return 0;
+  }
+
   PRInt32 rv = PR_Write(tls->ssl, cp, (int)n);
   // log_debug(LD_NET, "PR_Write(%zu) returned %d", n, (int)rv);
   if (rv > 0) {
@@ -646,13 +650,6 @@ tor_tls_block_renegotiation(tor_tls_t *tls)
   /* We don't support renegotiation with NSS. */
 }
 
-void
-tor_tls_assert_renegotiation_unblocked(tor_tls_t *tls)
-{
-  tor_assert(tls);
-  /* We don't support renegotiation with NSS. */
-}
-
 int
 tor_tls_get_pending_bytes(tor_tls_t *tls)
 {
@@ -791,17 +788,6 @@ tor_tls_cert_matches_key,(const tor_tls_t *tls,
   return rv;
 }
 
-MOCK_IMPL(int,
-tor_tls_get_tlssecrets,(tor_tls_t *tls, uint8_t *secrets_out))
-{
-  tor_assert(tls);
-  tor_assert(secrets_out);
-
-  /* There's no way to get this information out of NSS. */
-
-  return -1;
-}
-
 MOCK_IMPL(int,
 tor_tls_export_key_material,(tor_tls_t *tls, uint8_t *secrets_out,
                              const uint8_t *context,
