@@ -329,18 +329,15 @@ test_protover_supports_version(void *arg)
  * headers. */
 #define PROTOVER_LINKAUTH_V1 1
 #define PROTOVER_LINKAUTH_V2 2
-#define PROTOVER_RELAY_V1 1
+#define PROTOVER_RELAY_V2 2
 
 /* Deprecated HSIntro versions */
 #define PROTOVER_HS_INTRO_DEPRECATED_1 1
 #define PROTOVER_HS_INTRO_DEPRECATED_2 2
-/* Highest supported HSv2 introduce protocol version.
- * It's not clear if we actually support version 2, see #25068. */
-#define PROTOVER_HS_INTRO_V2 3
 
 /* HSv2 Rend and HSDir protocol versions. */
 #define PROTOVER_HS_RENDEZVOUS_POINT_V2 1
-#define PROTOVER_HSDIR_V2 1
+#define PROTOVER_HSDIR_V2 2
 
 /* DirCache, Desc, Microdesc, and Cons protocol versions. */
 #define PROTOVER_DIRCACHE_V1 1
@@ -359,6 +356,8 @@ test_protover_supports_version(void *arg)
 
 #define PROTOVER_FLOWCTRL_V1 1
 
+#define PROTOVER_RELAY_NTOR_V3 4
+
 /* Make sure we haven't forgotten any supported protocols */
 static void
 test_protover_supported_protocols(void *arg)
@@ -398,7 +397,7 @@ test_protover_supported_protocols(void *arg)
   /* Relay protovers do not appear anywhere in the code. */
   tt_assert(protocol_list_supports_protocol(supported_protocols,
                                             PRT_RELAY,
-                                            PROTOVER_RELAY_V1));
+                                            PROTOVER_RELAY_V2));
   tt_assert(protocol_list_supports_protocol(supported_protocols,
                                             PRT_RELAY,
                                             PROTOVER_RELAY_EXTEND2));
@@ -419,12 +418,6 @@ test_protover_supported_protocols(void *arg)
   tt_assert(!protocol_list_supports_protocol(supported_protocols,
                                             PRT_HSINTRO,
                                             PROTOVER_HS_INTRO_DEPRECATED_2));
-  /* We could test legacy HSIntro by calling rend_service_update_descriptor(),
-   * and checking the protocols field. But that's unlikely to change, so
-   * we just use a hard-coded value. */
-  tt_assert(protocol_list_supports_protocol(supported_protocols,
-                                            PRT_HSINTRO,
-                                            PROTOVER_HS_INTRO_V2));
   /* Test for HSv3 HSIntro */
   tt_assert(protocol_list_supports_protocol(supported_protocols,
                                             PRT_HSINTRO,
@@ -653,7 +646,8 @@ test_protover_vote_roundtrip_ours(void *args)
             "supports_establish_intro_dos_extension: %d,\n" \
             "supports_v3_hsdir: %d,\n" \
             "supports_v3_rendezvous_point: %d,\n" \
-            "supports_hs_setup_padding: %d.", \
+            "supports_hs_setup_padding: %d,\n" \
+            "supports_congestion_control: %d.", \
             (flags).protocols_known, \
             (flags).supports_extend2_cells, \
             (flags).supports_accepting_ipv6_extends, \
@@ -665,7 +659,8 @@ test_protover_vote_roundtrip_ours(void *args)
             (flags).supports_establish_intro_dos_extension, \
             (flags).supports_v3_hsdir, \
             (flags).supports_v3_rendezvous_point, \
-            (flags).supports_hs_setup_padding); \
+            (flags).supports_hs_setup_padding, \
+            (flags).supports_congestion_control); \
     STMT_END
 
 /* Test that the proto_string version version_macro sets summary_flag. */
